﻿using System;
using System.Data;
using System.Collections.Generic;
using System.IO;
using System.Linq;

namespace TestForLinq
{
    static class Program
    {
        private static decimal RandomAmount(Random rnd)
        {
            return Math.Round(Convert.ToDecimal(rnd.NextDouble() * 10000), 2);
        }

        static void Main(string[] args)
        {
            Console.WriteLine("Kreiranje sample podataka");
            MainDataSet ds = new MainDataSet();
            Random rnd = new Random();
            for (int x = 0; x < 5000; x++)
            {
                MainDataSet.KlijentRow r = ds.Klijent.AddKlijentRow(Path.GetRandomFileName(),
                                                                    Path.GetRandomFileName() + Path.GetRandomFileName());
                for (int y = 0; y < 6; y++) ds.Uplata.AddUplataRow(r, RandomAmount(rnd), DateTime.Now);
                for (int y = 0; y < 12; y++) ds.Zaduzenje.AddZaduzenjeRow(r, RandomAmount(rnd), DateTime.Now);
            }



            //klasican foreach
            Console.WriteLine(":: Start (FOREACH)");
            DateTime start = DateTime.Now;
            foreach (MainDataSet.KlijentRow krow in ds.Klijent)
            {
                decimal sumU = 0, sumD = 0;
                foreach (MainDataSet.UplataRow uplataRow in krow.GetUplataRows()) sumU += uplataRow.Iznos;
                foreach (MainDataSet.ZaduzenjeRow zaduzenjeRow in krow.GetZaduzenjeRows()) sumD += zaduzenjeRow.Iznos;
                ds.Rezultat.AddRezultatRow(krow.Ime, krow.Prezime, sumU, sumD, sumD - sumU);
            }
            Console.WriteLine(":: Stop  (FOREACH), Rows: {0} Time: {1}", ds.Rezultat.Count, DateTime.Now - start);

            // new run
            ds.Rezultat.Clear();
            Console.WriteLine(":: Start (LINQ)");
            start = DateTime.Now;
            // preko dva subquerya
            var final = from k in ds.Klijent
                        join uu in
                            (from u in ds.Uplata
                             group u by u.IDKupac
                                 into uu
                                 select new { IDKupac = uu.Key, UkupnoUplata = uu.Sum(x => x.Iznos) })
                            on k.ID equals uu.IDKupac into ug
                        join uz in
                            (from u in ds.Zaduzenje
                             group u by u.IDKupac
                                 into uz
                                 select new { IDKupac = uz.Key, UkupnoZaduzenje = uz.Sum(x => x.Iznos) })
                            on k.ID equals uz.IDKupac into zg
                        from uglo in ug.DefaultIfEmpty()
                        from zglo in zg.DefaultIfEmpty()
                        select
                            new
                                {
                                    k.ID,
                                    k.Ime,
                                    k.Prezime,
                                    UkupnoUplata = uglo == null ? 0 : uglo.UkupnoUplata,
                                    UkupnoZaduzenje = zglo == null ? 0 : zglo.UkupnoZaduzenje
                                };
            foreach (var e in final)
                ds.Rezultat.AddRezultatRow(e.Ime, e.Prezime, e.UkupnoUplata, e.UkupnoZaduzenje,
                                           e.UkupnoZaduzenje - e.UkupnoUplata);

            //rez.CopyToDataTable(ds.Rezultat);
            Console.WriteLine(":: Stop  (LINQ), Rows: {0} Time: {1}", ds.Rezultat.Count, DateTime.Now - start);



            Console.ReadLine();
        }
    }
}
